#include "Shader.hpp"

#include <stdio.h>
#include <string.h>
#include <iostream>

#include <GL/glew.h>

#include "../Utils/GLUtils.hpp"

char Shader :: checkShaderCompilation(const std::string& shaderName, GLuint shaderID)
{
    GLint compilationStatus = 0;

    /* Verification de la compialtion pour le vertex shader */
    glGetShaderiv(shaderID, GL_COMPILE_STATUS, &compilationStatus);
    if ( compilationStatus != GL_TRUE )
    {
        /* Nous savons que la compilation a chou, donc nous devons savoir pourquoi */
        /* Nous devons rcupr la taille du log */
        GLint logLength = 0;

        glGetShaderiv(shaderID, GL_INFO_LOG_LENGTH, &logLength);

        /* Maintenant que nous avons la taille, nous pouvons allou la mmoire suffisante pour ce log */
        GLchar* log = new GLchar[logLength * sizeof(GLchar)];

        glGetShaderInfoLog(shaderID, logLength, &logLength, log);

        /* On peut afficher le message */
        std::cout << "Fail to link the shader '" << shaderName << "': " << log << std::endl;

        delete [] log;

        return 0;
    }

    return 1; /* Pas d'erreur */
}

unsigned int Shader :: getFileSize(FILE* const pFile)
{
    long length = 0;

    fseek(pFile,0,SEEK_END);

    length = ftell(pFile);

    // Ne pas oublier de mettre le fichier  son dbut, sinon on ne peut pas le lire
    fseek(pFile,0,SEEK_SET);

    return length;
}

/**
    Le fonction retourne un pointeur, qui a t allou dans la fonction mme.
    ! Ceci est extrmement dangereux car cela peut entrainer des fuites de mmoire
    du  l'oubli de libr le pointeur.
*/
char* Shader :: readFile(const std::string& fileName)
{
    FILE* pFile = NULL;
    char* fileContent = NULL;
    unsigned int fileSize = 0;

    pFile = fopen(fileName.c_str(),"r");
    if ( pFile == NULL )
    {
        std::cerr << "Erreur d'ouverture du fichier: '" << fileName << "'" << std::endl;
        return NULL;
    }

    fileSize = getFileSize(pFile);

    fileContent = new char[fileSize+1];
    if ( fileContent == NULL )
    {
        std::cerr << "Erreur d'allocation de la mmoire pour y placer le contenu du fichier\n" << std::endl;
        return NULL;
    }

    fread(fileContent,fileSize,1,pFile);
    // Termine le tableau qui contient le shader
    fileContent[fileSize] = '\0';

    fclose(pFile);

    return fileContent;
}

Shader :: Shader(const std::string& vertexShaderFileName, const std::string& pixelShaderFile)
{
    vsid = glCreateShader(GL_VERTEX_SHADER);
    psid = glCreateShader(GL_FRAGMENT_SHADER);

    GLUtils::checkError("glCreateShader");

    GLchar* dataVertexShader = (GLchar*)readFile(vertexShaderFileName);
    GLchar* dataPixelShader = (GLchar*)readFile(pixelShaderFile);

    if ( !dataVertexShader || !dataPixelShader )
    {
        // Ici, il faudrait faire en sorte que le programme s'arrte
        throw ShaderException("Fail to read shaders");
        return;
    }

    // Chargement des sources dans OpenGL
    GLint vertexSize = strlen(dataVertexShader);
    GLint fragmentSize = strlen(dataPixelShader);
    glShaderSource(vsid, 1, (const GLchar**)(&dataVertexShader), &vertexSize);
    glShaderSource(psid, 1, (const GLchar**)(&dataPixelShader), &fragmentSize);

    // Compilation du vertex shader
    glCompileShader(vsid);
    glCompileShader(psid);

    // Vrification des erreurs
    if ( !checkShaderCompilation(vertexShaderFileName,vsid) || !checkShaderCompilation(pixelShaderFile,psid))
    {
        throw ShaderCreationException();
        return;
    }

    // Creation de l'ID pour le programme
    pgid = glCreateProgram();

    // On attache les shader ensemble
    glAttachShader(pgid, vsid);
    glAttachShader(pgid, psid);

    // On peut enfin passer aux liage.
    glLinkProgram(pgid);

    // Et encore une fois on vrifie si tout se passe bien
    GLint programState = 0;
    glGetProgramiv(pgid , GL_LINK_STATUS  , &programState);
    if ( programState != GL_TRUE)
    {
        // On rcupre la taille du log
        GLint logSize = 0;

        glGetProgramiv(pgid, GL_INFO_LOG_LENGTH, &logSize);

        // On peut allouer la mmoire, une fois que l'on a la taille du log
        GLchar* log = new GLchar[logSize];
        if ( log == NULL )
        {
            fprintf(stderr,"Erreur d'allocation de mmoire pour le log de la compilation du programme\n");
            throw ShaderException("Fail to allocate memory for linker log");
            return;
        }

        // Et on rcupre le log
        glGetProgramInfoLog(pgid, logSize, &logSize, log);

        // On affiche
        fprintf(stderr,"Erreur lors du liage du shader:\n%s",log);

        delete [] log;
        throw ShaderLoadingException(vertexShaderFileName + std::string(" ") + pixelShaderFile + std::string(" Linking"));
        return;
    }

    delete dataPixelShader;
    delete dataVertexShader;
}

Shader :: ~Shader()
{
    /* On arrte d'utiliser le programme shader */
    glUseProgram(0);

    /* Deliage des shaders au programme */
    glDetachShader(pgid, psid);
    glDetachShader(pgid, vsid);

    /* Destruction du programme */
    glDeleteProgram(pgid);

    /* Destruction des IDs des shaders */
    glDeleteShader(psid);
    glDeleteShader(vsid);
}

void Shader :: enable()
{
    glUseProgram(pgid);
}

void Shader :: disable()
{
    glUseProgram(0);
}

GLint Shader :: getProgram()
{
    return pgid;
}
