#include "Grid.hpp"

#include <stdlib.h>

#include "../Types/Vector3.hpp"
#include "../Types/Vector4.hpp"

#include "../Utils/MathUtils.hpp"

const Colour4 gridColour = Colour4(117.0f/255.0f,117.0f/255.0f,117.0f/255.0f,0.7f);

Grid :: Grid(unsigned int numberPointsX, unsigned int numberPointsY, float step):numberPointsX(numberPointsX),numberPointsY(numberPointsY),hadImpulsion(numberPointsX * numberPointsY,0),impulsionUpdateStep(0),colours(numberPointsX * numberPointsY, gridColour),impulsionValue(15)
{
    unsigned int nbPoints = numberPointsX * numberPointsY;

    for ( unsigned int j = 0 ; j < 16 ; j ++ )
    {
        std::vector < unsigned int > pics;
        for ( unsigned int i = 0 ; i < 64 ; i++ )
        {
            pics.push_back(rand()%nbPoints);
        }

        picsSpectrum.push_back(pics);
    }

    points.reserve(nbPoints);

    for ( unsigned int x = 0 ; x < numberPointsX ; x++ )
    {
        for ( unsigned int y = 0 ; y < numberPointsY ; y++ )
        {
            points.push_back(Point(Vector3f(x*step, 0, y*step), gridColour));
        }
    }

    unsigned int i = 0;
    for ( unsigned int x = 0 ; x < numberPointsX ; x++ )
    {
        for ( unsigned int y = 0 ; y < numberPointsY-1 ; y++ )
        {
            indices.push_back(y + x * numberPointsY);
            indices.push_back(y + x * numberPointsY + 1);
        }
    }

    i = 0;
    for ( unsigned int y = 0 ; y < numberPointsY ; y++ )
    {
        for ( unsigned int x = 0 ; x < numberPointsX-1 ; x++ )
        {
            indices.push_back(y + x * numberPointsY);
            indices.push_back(y + (x+1) * numberPointsY);
        }
    }
}

void Grid :: makeElevation(unsigned int x, unsigned int y, float height, int displacement)
{
    if ( x > numberPointsX || y > numberPointsY || height <= 0 || displacement > 5)
    {
        return;
    }

    unsigned int index = x + y*numberPointsX;
    if ( points[index].position.y == 0 )
    {
        points[index].position.y = height;
    }

    float newElevation = height - displacement;
    //float newElevation = -((displacement+1) * (displacement+1)) + height;
    makeElevation(x-1,y,newElevation,displacement+1);
    makeElevation(x,y-1,newElevation,displacement+1);
    makeElevation(x,y+1,newElevation,displacement+1);
    makeElevation(x+1,y,newElevation,displacement+1);
}

void Grid :: makeImpulsion(unsigned int x, unsigned int y, const Colour4& colour, unsigned int impulsionStep, unsigned int limit)
{
    unsigned int index = x + y*numberPointsX;

    if ( x >= numberPointsX || y >= numberPointsY || impulsionStep >= limit || (hadImpulsion[index] <= impulsionStep && hadImpulsion[index] != 0) )
    {
        return;
    }
    if ( impulsionStep == 0 )
    {
        impulsionStep = 1;
    }

    hadImpulsion[index] = impulsionStep;
    colours[index] = colour;

    makeImpulsion(x-1,y,colour,impulsionStep+1,limit);
    makeImpulsion(x+1,y,colour,impulsionStep+1,limit);
    makeImpulsion(x,y+1,colour,impulsionStep+1,limit);
    makeImpulsion(x,y-1,colour,impulsionStep+1,limit);

    impulsionUpdateStep = 1;
}

void Grid :: updateSpectrum(float* spectrum, const unsigned int precision)
{
    for ( unsigned int i = 0 ; i < 16 ; i++ )
    {
        for ( std::vector<unsigned int>::iterator itPics = picsSpectrum[i].begin() ; itPics != picsSpectrum[i].end() ; ++itPics )
        {
            if ( points[*itPics].position.y > 0.4f )
            {
                points[*itPics].position.y-=0.1;
            }
            else
            {
                points[*itPics].position.y = 0;
                points[*itPics].colour = colours[i];
            }

            if ( points[*itPics].position.y < spectrum[i] * 15 )
            {
                points[*itPics].position.y = spectrum[i] * impulsionValue;
                points[*itPics].colour = colours[i] * 0.6;
            }
        }
        /*
        points[indicesSpectrum[i]].position.y = spectrum[i] * 5;
        points[indicesSpectrum[i]].colour.b = spectrum[i] * 20;
        */
    }
}

void Grid :: updateImpulsion()
{
    if ( impulsionUpdateStep != 0 )
    {
        for ( unsigned int i = 0 ; i < points.size() ; i++ )
        {
            if ( hadImpulsion[i] == impulsionUpdateStep )
            {
                points[i].colour = Colour4(1.0,1.0,1.0,0.7);
            }
            else if ( hadImpulsion[i] < impulsionUpdateStep )
            {
                points[i].colour = MathUtils::lerp(points[i].colour,colours[i],0.1);
                if ( points[i].colour.r - gridColour.r < 0.05f && points[i].colour.g - gridColour.g < 0.05f && points[i].colour.b - gridColour.b < 0.05f )
                {
                    points[i].colour = colours[i];
                    hadImpulsion[i] = 0;
                }
            }
        }

        impulsionUpdateStep++;

        bool endImpulsion = true;
        for ( unsigned int i = 0 ; i < hadImpulsion.size() ; i++ )
        {
            if ( hadImpulsion[i] != 0 )
            {
                endImpulsion = false;
                break;
            }
        }

        if ( endImpulsion == true )
        {
            impulsionUpdateStep = 0;
        }
    }
}

void Grid :: clearColour()
{
    for ( std::vector<Point>::iterator itPoint = points.begin() ; itPoint != points.end() ; ++itPoint)
    {
        itPoint->colour = gridColour;
    }

    for ( std::vector<Colour4>::iterator itColour = colours.begin() ; itColour != colours.end() ; ++itColour)
    {
        (*itColour) = gridColour;
    }
}

void Grid :: clearColour(const Colour3& colour)
{
    for ( std::vector<Point>::iterator itPoint = points.begin() ; itPoint != points.end() ; ++itPoint)
    {
        itPoint->colour = colour;
    }

    for ( std::vector<Colour4>::iterator itColour = colours.begin() ; itColour != colours.end() ; ++itColour)
    {
        (*itColour) = colour;
    }
}

void Grid :: clearElevation()
{
    for ( std::vector<Point>::iterator itPoint = points.begin() ; itPoint != points.end() ; ++itPoint)
    {
        itPoint->position.y = 0;
    }
}
